//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Loaders/QREDataLoaderProperties.cpp
//! @brief     Implements class AutomaticMultiColumnDataLoader1DProperties
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Loaders/QREDataLoaderProperties.h"
#include <QCheckBox>
#include <QComboBox>
#include <QFormLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpinBox>
#include <QVBoxLayout>
#include <QtGui>

QREDataLoaderProperties::QREDataLoaderProperties()
    : m_allowFactors(false)
{
    setGeometry(0, 0, 486, 241);
    setWindowTitle("Form");

    auto vbox = new QVBoxLayout;
    setLayout(vbox);

    m_gridLayout = new QGridLayout;
    vbox->addLayout(m_gridLayout);

    m_enableErrorCheckBox = new QCheckBox;
    m_gridLayout->addWidget(m_enableErrorCheckBox, 2, 0);
    m_gridLayout->addWidget(new QLabel("Read Q"), 0, 1);
    m_gridLayout->addWidget(new QLabel("Read R"), 1, 1);
    m_gridLayout->addWidget(new QLabel("Read σR"), 2, 1);

    for (int i = 0; i < 3; ++i) {
        m_gridLayout->addWidget(new QLabel("from  column"), i, 2);
        auto sb = new QSpinBox;
        m_gridLayout->addWidget(sb, i, 3);
        sb->setMinimum(1);
    }

    auto w21 = new QWidget;
    vbox->addWidget(w21);

    auto form211 = new QFormLayout;
    w21->setLayout(form211);

    m_separatorCombo = new QComboBox;
    form211->addRow("Separator:", m_separatorCombo);
    m_separatorCombo->addItem("TAB");
    m_separatorCombo->addItem("SPACE");
    m_separatorCombo->addItem(",");
    m_separatorCombo->addItem(";");

    m_linesToSkipEdit = new QLineEdit;
    m_linesToSkipEdit->setPlaceholderText("Example: 1-5, 12");
    form211->addRow("Ignore line numbers:", m_linesToSkipEdit);

    m_headerPrefixEdit = new QLineEdit;
    m_headerPrefixEdit->setPlaceholderText("#,//");
    form211->addRow("Ignore lines starting with:", m_headerPrefixEdit);

    m_qUnitCombo = new QComboBox;
    form211->addRow("Q given in units of:", m_qUnitCombo);
    m_qUnitCombo->addItem("1/nm");
    m_qUnitCombo->addItem("1/Å");

    allowFactors(false);

    connect(m_headerPrefixEdit, &QLineEdit::textChanged, [this]() { emit propertiesChanged(); });
    connect(m_linesToSkipEdit, &QLineEdit::textChanged, [this]() { emit propertiesChanged(); });
    connect(m_separatorCombo, &QComboBox::currentTextChanged,
            [this]() { emit propertiesChanged(); });

    connect(m_enableErrorCheckBox, &QCheckBox::stateChanged, this,
            &QREDataLoaderProperties::onErrorEnablingChanged);

    connect(m_qUnitCombo, QOverload<int>::of(&QComboBox::currentIndexChanged),
            [this]() { emit propertiesChanged(); });

    for (int i = 0; i < 3; i++) {
        // In the following: disable the checkbox before signaling propertyChanged to suppress ghost
        // value changes in case the propertiesChanged signals triggers long calculations (the
        // spinbox would see a timeout in its "pressed"-loop and generate a second value change)
        connect(columnSpinBox(i), QOverload<int>::of(&QSpinBox::valueChanged), [this, i]() {
            QSpinBox* sb = columnSpinBox(i);
            if (sb)
                sb->setEnabled(false);
            emit propertiesChanged();
            if (sb)
                sb->setEnabled(true);
        });

        connect(factorSpinBox(i), QOverload<double>::of(&QDoubleSpinBox::valueChanged),
                [this, i]() {
                    QDoubleSpinBox* sb = factorSpinBox(i);
                    if (sb)
                        sb->setEnabled(false);
                    emit propertiesChanged();
                    if (sb)
                        sb->setEnabled(true);
                });
    }
}

void QREDataLoaderProperties::allowFactors(bool b)
{
    m_allowFactors = b;
    for (int dataType = 0; dataType < 3; dataType++) {
        if (auto* label = factorLabel(dataType))
            label->setVisible(b);
        if (auto* sb = factorSpinBox(dataType))
            sb->setVisible(b);
    }

    updateErrorEnabling(m_enableErrorCheckBox->isChecked());
}

double QREDataLoaderProperties::factor(int dataType) const
{
    auto* const sb = factorSpinBox(dataType);
    return sb && sb->isVisible() ? sb->value() : 1.0;
}

void QREDataLoaderProperties::updateErrorEnabling(bool enabled) const
{
    const int lineInLayout = 2;

    for (int col = 2; col < m_gridLayout->columnCount(); col++) {
        auto* const layoutItem = m_gridLayout->itemAtPosition(lineInLayout, col);
        if (layoutItem) {
            QWidget* w = layoutItem->widget();
            if (w) {
                const bool belongsToFactor = col == factorColumn || col == factorLabelColumn;
                w->setVisible(enabled && (!belongsToFactor || m_allowFactors));
            }
        }
    }
}

void QREDataLoaderProperties::onErrorEnablingChanged()
{
    const bool isEnabled = m_enableErrorCheckBox->isChecked();

    updateErrorEnabling(isEnabled);

    emit propertiesChanged();
}

QSpinBox* QREDataLoaderProperties::columnSpinBox(int dataType) const
{
    const int lineInLayout = dataType;
    QLayoutItem* item = m_gridLayout->itemAtPosition(lineInLayout, columnColumn);
    if (!item)
        return {};
    return dynamic_cast<QSpinBox*>(item->widget());
}

QDoubleSpinBox* QREDataLoaderProperties::factorSpinBox(int dataType) const
{
    const int lineInLayout = dataType;
    QLayoutItem* item = m_gridLayout->itemAtPosition(lineInLayout, factorColumn);
    if (!item)
        return {};
    return dynamic_cast<QDoubleSpinBox*>(item->widget());
}

QLabel* QREDataLoaderProperties::factorLabel(int dataType) const
{
    const int lineInLayout = dataType;
    QLayoutItem* item = m_gridLayout->itemAtPosition(lineInLayout, factorLabelColumn);
    if (!item)
        return {};
    return dynamic_cast<QLabel*>(item->widget());
}
